import logging

import gettext
from gettext import gettext as _
gettext.textdomain('unity-lens-tomboy')

from singlet.lens import SingleScopeLens, IconViewCategory, ListViewCategory
from gi.repository import GLib, Gio, Unity

from unity_lens_tomboy import unity_lens_tomboyconfig


class TomboyLens(SingleScopeLens):

    class Meta:
        name = 'tomboy'
        description = 'Tomboy Lens'
        search_hint = _('Search Tomboy Notes')
        icon = 'tomboy.svg'
        search_on_blank = True

    notes_category = IconViewCategory(_('Tomboy Notes'), 'tomboy-panel')

    notebook_filter = Unity.RadioOptionFilter.new('notebook', _('Notebook'),
                                                  None, False)

    def __init__(self):
        super(TomboyLens, self).__init__()
        self.tomboy = None
        session_bus_connection = Gio.bus_get_sync(Gio.BusType.SESSION, None)
        Gio.DBusProxy.new(session_bus_connection, 0, None,
                                        'org.gnome.Tomboy',
                                        '/org/gnome/Tomboy/RemoteControl',
                                        'org.gnome.Tomboy.RemoteControl', None,
                                        self.tomboy_ready, None)

    def tomboy_ready(self, proxy, result, name):
        try:
            self.tomboy = Gio.DBusProxy.new_finish(result)
        except GLib.GError:
            # Mostly timeouts while waiting for tomboy.
            # Quit in these cases, the lens will be restarted as needed.
            logging.error(u'Cannot connet to tomboy. Exiting.')
            import sys
            sys.exit(1)
        # We cannot use _scope.get_filter here, because that gives us
        # a read-only filter instance.
        notebook_filter = self._meta.filter_dict['notebook_filter']
        for notebook in self.list_all_notebooks():
            display_name = notebook.decode('utf8').split(':', 2)[2].title()
            logging.debug(u'Adding notebook {} to filter'.format(display_name))
            notebook_filter.add_option(notebook, display_name, None)

    def list_all_notebooks(self):
        notebooks = set()
        for note in self.tomboy.ListAllNotes():
            tags = self.tomboy.GetTagsForNote('(s)', note)
            note_notebooks = [tag for tag in tags
                              if tag.startswith('system:notebook:')]
            notebooks.update(note_notebooks)
        notebooks = sorted(notebooks, key=str.lower)
        return notebooks

    def search(self, search, results):
        if not self.tomboy:
            # Tomboy not ready yet.
            return
        notebook_filter = self._scope.get_filter('notebook')
        # get_active_options returns an Option instance, we want the
        # plain string or None
        active_notebook = notebook_filter.get_active_option()
        if active_notebook:
            active_notebook = active_notebook.props.id
        if search:
            logging.debug(u'Searching notes for {}'.format(search))
            notes = self.tomboy.SearchNotes('(sb)', search, False)
            if active_notebook:
                notes = [n for n in notes
                         if active_notebook
                         in self.tomboy.GetTagsForNote('(s)', n)]
        else:
            if active_notebook:
                logging.debug(u'List all notes for notebook {}'.format(
                              active_notebook))
                notes = self.tomboy.GetAllNotesWithTag('(s)', active_notebook)
            else:
                logging.debug(u'List all notes.')
                notes = self.tomboy.ListAllNotes()
        for note_uri in notes:
            results.append(
                note_uri,
                'tomboy',
                self.notes_category,
                'application/x-desktop',
                self.tomboy.GetNoteTitle('(s)', note_uri),
                self.tomboy.GetNoteContents('(s)', note_uri),
                note_uri)

    def handle_uri(self, scope, uri):
        self.tomboy.DisplayNote('(s)', uri)
        return self.hide_dash_response(uri)
